local ADDON_NAME, Core = ...

--<
local date = date
local table = table
local select = select
local string = string
local UI = Core.UI
local Utils = Core.utils
-->

local tabTraverseIndex = 0
local playerCharacter
local playerRealm
local playerName


local App = {
	active = {},
}
local Session = {}
Session.__index = Session
local cache = {}
local lastWhisperTarget
local selectedSession
local temporaryPointer


function App:AddChannel (channel)
	local typeID = string.upper(channel)
	
	self.active[typeID] = Session:CreateFromChannel(channel)
	self:Get(typeID):Init("CHANNEL")
	
	return self:Get(typeID)
end


function App:AddCustomChannel (channel)
	local typeID = string.upper(channel)
	
	self.active[typeID] = Session:New(channel)
	self.active[typeID]:Init("CHANNEL")
	
	C_Timer.After(3, function() App:Get("NEARBY"):UpdateTitle() end)	
	
	return self:Get(typeID)
end


function App:Switch (to)
	if (selectedSession ~= nil) and (selectedSession ~= to) then
		selectedSession:Deselect()
		selectedSession.savedText = UI.editBox:GetText()
		
		to:SetSelected()
		to:StopGlowing()
		
		selectedSession = to
		tabTraverseIndex = to.tab.tabIndex
		
		UI:UpdateHeader(selectedSession)
		UI.editBox:SetTextColor(selectedSession:GetRGB())
	
	elseif (selectedSession == to) then
		UI.editBox:SetFocus()
	end
end


function App:Get (tag)
	return self.active[tag] or false
end


function App:SendWho (who)
	C_FriendList.SetWhoToUi(true)
	FriendsFrame:UnregisterEvent("WHO_LIST_UPDATE")
	C_FriendList.SendWho(who)
end


function App:OnWhoReceived ()
	for i=1, C_FriendList.GetNumWhoResults() do
		local info = C_FriendList.GetWhoInfo(i)
		local S = App:Get(info.fullName)

		if S then
			S:UpdateLevel(info.level)
		end
	end
	
	C_FriendList.SetWhoToUi(false)
	FriendsFrame:RegisterEvent("WHO_LIST_UPDATE")
end



-------------------------------------------------------------------
-------------------------------------------------------------------



function Session:New (...)
	local tbl = table.remove(cache) or {}
	setmetatable(tbl, self)
	
	local channel, fullName = ...
	
	if fullName then
		tbl.targetID = fullName
		tbl.chatType = "WHISPER"
		tbl.title = fullName
		tbl.tab = Core.CreateNewTab(channel, "TEMPORARY")
		tbl.page = Core.CreateViewerPage(false)
	else
		tbl.targetID = "SAY"
		tbl.chatType = "SAY"
		tbl.title = "Say".. " (".. GetMinimapZoneText().. ")"
		tbl.tab = Core.CreateNewTab(channel, channel)
		tbl.page = Core.CreateViewerPage(false)
		tbl.isCustom = true
	end
	
	return tbl
end

function Session:CreateFromChannel (channel)
	local tbl = table.remove(cache) or {}
	setmetatable(tbl, self)
	
	tbl.targetID = channel:upper()
	tbl.chatType = channel:upper()
	tbl.title = channel
	tbl.isChannel = true
	
	tbl.tab = Core.CreateNewTab(channel, channel)
	tbl.page = Core.CreateViewerPage(false)
	
	return tbl
end

function Session:CreateFromID (id, isBnet)
	local tbl = table.remove(cache) or {}
	setmetatable(tbl, self)
	
	local payload
	tbl.isDM = true
	tbl.typeInform_lastSent = 0
	tbl.typeInform_lastUpdated = 0
	
	if isBnet then
		payload = C_BattleNet.GetAccountInfoByID(id)
		tbl.targetID = payload.bnetAccountID
		tbl.identifier = payload.battleTag
		tbl.chatType = "BN_WHISPER"
		tbl.altName = payload.battleTag:upper()
		tbl.title = payload.accountName
		
	else
		payload = {GetPlayerInfoByGUID(id)}
		if #payload[7] > 0 then 
			tbl.targetID = payload[6] .. "-" .. payload[7]
			tbl.title = payload[6].. " (" .. payload[7] .. ")"
		else 
			tbl.targetID = payload[6] .. "-" .. playerRealm
			tbl.title = payload[6]
		end
		
		tbl.identifier = id
		tbl.chatType = "WHISPER"
		tbl.altName = tbl.targetID:upper()
	end
	
	tbl.tab = Core.CreateNewTab(payload, isBnet)
	tbl.page = Core.CreateViewerPage(true)

	return tbl
end

function Session:Init (mode)
	self.tab:SetScript("OnMouseDown", function(this, b)
		if b == "LeftButton" then
			App:Switch(self)
			
		elseif b == "RightButton" then
			self:ShowMenu()
		end			
	end)

	self.savedText = ""
	self.hasUnread = true
	
	if selectedSession == nil then
		selectedSession = self
		self:SetSelected()
		UI:UpdateHeader(selectedSession)
		UI.editBox:SetTextColor(self:GetRGB())
		
	else
		self:Deselect()
	end
	
	if mode == "CHANNEL" then return end
	
	if not UI:IsEventRegistered("CHAT_MSG_ADDON") then
		UI:RegisterEvent("CHAT_MSG_ADDON")
	end
	
	if type(self.targetID) ~= "number" then
		self:CommCheck("OUT")
	end

	if not Core.History:NewEntry(self.altName, self.identifier) then
		local latest, datestring = Core.History:GetMostRecentDate(self.altName)
		
		if latest then
			local context = {}
			
			self.page:AddMessage("~~~~~ Start of history: ".. datestring)
			
			for i=#latest-5, #latest do
				if latest[i] ~= nil then
					context = {
						inbound = latest[i].IN or latest[i].inbound,
						msg = latest[i].MSG or latest[i].content,
						chatTimeStamp = latest[i].TS or latest[i].timestamp,
					}
					
					if not context.inbound then context.author = "Me" else context.author = self:GetName() end
					
					self:AddMsg(context)
				end
			end
			
			self.page:AddMessage("~~~~~ End of history.")
		end
	end
	
	self.link = SESMSG_History[self.altName]

end

function Session:GetRGB ()
	local c = ChatTypeInfo[self.chatType]
	return c.r, c.g, c.b
end

function Session:GetName ()
	return self.tab.sessionName:GetText()
end

function Session:GetID ()
	return self.targetID
end

function Session:CommCheck (way)
	if way == "OUT" then
		C_ChatInfo.SendAddonMessage("SessionsMSGR", "SMCHK", "WHISPER", self.targetID)
		
	elseif way == "IN" then
		self.isUsingSM = true
	end
end

function Session:UpdatePortrait (unit)
	SetPortraitTexture(self.tab.portrait, unit)
end

function Session:UpdateLevel (lvl)
	local infoString = self.tab.infoString:GetText()
	local x, occurrence = infoString:find("%d+")

	if occurrence then
		infoString = infoString:sub(occurrence+2, #infoString)
	end
	
	self.tab.infoString:SetText(tostring(lvl).. " ".. infoString)
end

function Session:UpdateTitle (ctype)
	self.title = (ctype or Utils.capitalize(self.chatType)).. " (".. GetMinimapZoneText().. ")"
	
	if selectedSession ==  self then
		UI:UpdateHeader(self)
	end
end

function Session:StartGlowing ()
	if not self.isMuted then
		self.tab.glow:Show()
		self.tab.glow.animation:Play()
	end
end

function Session:StopGlowing ()
	self.tab.glow:Hide()
	self.tab.glow.animation:Stop() 
end

function Session:SetSelected ()
	self.tab:SetBackdropBorderColor(1, 1, 1, 1)
	self.page:Show()
	self.hasUnread = false
	
	if self.chatType == "EMOTE" and self.savedText == "" then
		UI.editBox:SetText(playerCharacter)
	else
		UI.editBox:SetText(self.savedText)
	end	
	
	UI.editBox:HighlightText()
	UI.editBox:SetFocus()
	
	C_Timer.After(0, function() UI:SetSliderValues() end)
end

function Session:Deselect ()
	self.tab:SetBackdropBorderColor(0, 0, 0, 1)
	self.page:Hide()
end

function Session:AddMsg (context)
	local ts = "|cFF"..Utils.RGBPercToHex(.7, .7, .7).. context.chatTimeStamp.. "|r "
	local r, g, b, prefix
	
	if context.inbound == nil or context.inbound == true then
		r, g, b = self:GetRGB()
		prefix = "|Hplayer:".. context.author.. "|h".. context.author.. "|h: "
		self.page:ClearTypingStatus()
	
	elseif context.inbound == false then
		r, g, b = .85, .85, .85
		prefix = context.author.. ": "
	end
	
	self.page:AddMessage(ts.. prefix.. context.msg, r, g, b)
	self.page:Update()
end

function Session:SetTyping ()
	if not(time() >= self.typeInform_lastUpdated) and not self.page:IsShown() then
		return
	end

	self.page.typeInform:SetText("|TInterface/CHATFRAME/UI-ChatWhisperIcon:16:16|t".. self:GetName().. " is typing...")
	self.page.typeInform:Show()
	self.page:SetPoint("BOTTOMLEFT", 5, 19)
	self.typeInform_lastUpdated = time()+3

	C_Timer.After(4, function() self.page:ClearTypingStatus() end)
end

function Session:ShowMenu ()
	if self.menu == nil then
		local name = self:GetName()
		local hide = (name == "Guild") or (name == "Party") or (name == temporaryPointer) or (self.isCustom) or false
		
		if not hide then
			name = "|cFF".. Utils.RGBPercToHex(self.tab.sessionName:GetTextColor()).. name.. "|r"
		end

		self.menu = {
			{text=name, isTitle=true, notCheckable=true},
			{
				text="Update Info",
				notCheckable=true,
				disabled=hide,
				func=function() App:SendWho(self:GetName()) end
			},
			{
				text="View History",
				notCheckable=true,
				disabled=hide,
				func=function() Core.History:Fetch(self.altName) end
			},
			{
				text="Add Friend",--|TInterface/FriendsFrame/UI-Toast-FriendRequestIcon:24:24|t
				notCheckable=true,
				disabled=hide,
				func=function() C_FriendList.AddFriend(self.altName) end
			},
			{text="|TInterface/COMMON/UI-TooltipDivider:8:144|t", notClickable=true, notCheckable=true},
			{
				text="Mute Notifications",
				isNotRadio=true,
				checked=function() return self.isMuted end,
				func=function() self.isMuted = not self.isMuted end,
			},
			{
				text="Ignore",--|TInterface/PaperDollInfoFrame/UI-GearManager-LeaveItem-Transparent:16:16|t
				isNotRadio=true,
				disabled=hide,
				checked=function() if not hide then return C_FriendList.IsIgnoredByGuid(self.identifier) end end,
				func=function() C_FriendList.AddOrDelIgnore(self.altName) end,
			},
			{text="|TInterface/COMMON/UI-TooltipDivider:8:144|t", notClickable=true, notCheckable=true},
			{text="Close Session", notCheckable=true, func=function() self:Deactivate() end},
		}
	end	
	
	EasyMenu(self.menu, UI.cardMenu, "cursor", 0, 0, "MENU")
	
end

function Session:AddHistory (context)
	if not self.link.INFO.MEM then
		local author
		if context.inbound then author = Ambiguate(self:GetName(), "short") else author = playerCharacter end
		
		self.link.INFO.MEM = {
			["MSG"] = author .. ": " .. context.msg,
			["DT"] = context.dt,
			["ON"] = playerCharacter
		}
	end	
	
	if self.link.HISTORY[context.dt] == nil then
		self.link.HISTORY[context.dt] = {}
	end
	
	local slot = self.link.HISTORY[context.dt]
	
	slot[#slot+1] = {
		IN = context.inbound,
		TS = context.chatTimeStamp,
		MSG = context.msg
	}

end

function Session:Deactivate ()
	self:Deselect()
	
	local ref = self:GetName()
	
	Core.CloseTab(self.tab)
	self.page:Clear()
	
	if #Core.orderedTabs == 0 then
		selectedSession = nil
		UI.editBox:SetText("")
		UI.editBox:SetTextColor(.8, .8, .8)
		UI:UpdateHeader(nil)
		
	else
		for N, S in pairs(App.active) do
			if S.tab.tabIndex == 1 then
				App:Switch(S)
				break
			end
		end
	end
	
	for N, S in pairs(App.active) do
		if S == self then
			App.active[N] = nil
			break
		end	
	end
	
	local DMTabsRemaining = false
	for N, S in pairs(App.active) do
		if S.isUsingSM then
			DMTabsRemaining = true
		end
	end
	
	if not DMTabsRemaining then UI:UnregisterEvent("CHAT_MSG_ADDON") end
	if ref == temporaryPointer then temporaryPointer = nil end
	table.insert(cache, wipe(self))
end



-------------------------------------------------------------------
-------------------------------------------------------------------



local function resetPointer ()
	if temporaryPointer ~= nil then
		App.active[temporaryPointer]:Deactivate()
		temporaryPointer = nil
	end
end


local function chatMsgWhisperHandler (e, ...)
	if e:match("BN_") then
		target = select(13, ...)
	else
		target = Ambiguate( select(2, ...), "short" )
	end
	
	local context = table.remove(cache) or {}
	context.chatTimeStamp = date("%I:%M")
	context.dt = date("%Y-%m-%d")
	context.msg = select(1, ...)
	context.inbound = (e:match("_INFORM") == nil and true or false)
	
	local S = App:Get(target)

	if not S or target == temporaryPointer then
		if not context.inbound and target == temporaryPointer then
			context.priority = true
		end
		
		resetPointer()
		
		if e:match("BN_") ~= nil then
			App.active[target] = Session:CreateFromID(target, true)

		else
			App.active[target] = Session:CreateFromID(select(12, ...), false)
		end
		
		App.active[target]:Init()
		S = App.active[target]
	end
	
	if context.inbound then
		context.author = S:GetName()
	else
		if e:match("BN_") then
			context.author = playerName
		else
			context.author = playerCharacter
		end
	end
	
	S:AddMsg(context)
	S:AddHistory(context)
	
	if UI:IsShown() then
		if context.priority then
			App:Switch(S)
			
		elseif selectedSession ~= S then
			S:StartGlowing()
			S.hasUnread = true
		end
		
	elseif not UI:IsShown() and InCombatLockdown() and context.inbound then
		Core.hub:Toast(context.msg, S)
		Core.hub:Notify()
	
	elseif not UI:IsShown() and not InCombatLockdown() then
		App:Switch(S)
		UI:Show()
	end	
	
	if (context.inbound or lastWhisperTarget == nil) and (not S.isMuted) then
		lastWhisperTarget = S
		PlaySound(3081, "Master")
		FlashClientIcon()			
	end

	if S.tab.tabIndex ~= 1 and not S.isMuted then
		table.insert(Core.orderedTabs, 1, table.remove(Core.orderedTabs, S.tab.tabIndex))
		Core.Tab_ReanchorAll()
	end
	
	table.insert(cache, wipe(context))

end


local function chatMsgPartyHandler (e, ...)
	local ts = "|cff".. Utils.RGBPercToHex(.7, .7, .7).. date("%I:%M").. "|r "
	local name = Ambiguate(select(2, ...), "short")
	local prefix = "|Hplayer:".. name.. "|h".. name.. "|h: "
	local r, g, b
	
	if e:match("LEADER") then
		r, g, b = ChatTypeInfo["PARTY_LEADER"].r, ChatTypeInfo["PARTY_LEADER"].g, ChatTypeInfo["PARTY_LEADER"].b
	else
		if select(2, ...) == playerCharacter then
			r, g, b = .85, .85, .85
		else
			r, g, b = ChatTypeInfo["PARTY"].r, ChatTypeInfo["PARTY"].g, ChatTypeInfo["PARTY"].b
		end
	end
	
	App.active["PARTY"].page:AddMessage(ts.. prefix.. select(1, ...), r, g, b)
end


local function chatMsgChannelHandler (e, ...)
	context = table.remove(cache) or {}
	context.chatTimeStamp = date("%I:%M")
	context.msg = select(1, ...)
	context.author = Ambiguate(select(2, ...), "short")

	local S = App:Get("GUILD")
	
	if not S then
		S = App:AddChannel("Guild")
	end
	
	S:AddMsg(context)
	
	if context.msg:match(playerCharacter) and selectedSession ~= S then
		S:StartGlowing()
		S.hasUnread = true
	end
	
	table.insert(cache, wipe(context))
end


local function chatMsgCustomHandler (e, ...)
	local S = App:Get("NEARBY")
	
	if not S then
		S = App:AddCustomChannel("Nearby")
	end
	
	name = Ambiguate(select(2, ...), "none")
	
	nameLink = Utils.getClassColouredNameLink(name, select(2, GetPlayerInfoByGUID( select(12, ...)) ))
	msg = select(1, ...)
	ts = "|cFF".. Utils.RGBPercToHex(.7, .7, .7).. date("%I:%M").. "|r "
	
	if e == "EMOTE" then
		if string.sub(msg, 1, 1) ~= "'" then
			msg = " ".. msg
		end
		
		msg = "* ".. nameLink.. msg
		
	elseif e == "TEXT_EMOTE" then
		if name == playerCharacter then
			nameLink = "|cFF".. Utils.RGBPercToHex(.8, .8, .8).. "You|r"
		end
		
		msg = msg:gsub(name, nameLink)

	else
		msg = nameLink.. ": ".. msg		
	end

	S.page:AddMessage(ts.. msg, ChatTypeInfo[e].r, ChatTypeInfo[e].g, ChatTypeInfo[e].b)
	S.page:Update()
end


local function chatMsgRestrictedHandler (...)
	err, target = ...	
	local S = App:Get(Ambiguate(target, "short"))
	
	if err == "ChatIgnoredHandler" and S then
		S.page:AddMessage("Could not deliver message!", 1, 0, 0)
	end
end


local function sendMessage (msg, session)
	local queue = table.remove(cache) or {}
	local _max = 255
	local target = session:GetID()	
	local ctype = session.chatType
	
	if ctype == "EMOTE" and msg:find(playerCharacter) == 1 then
		msg = Utils.splitString(msg, #playerCharacter+1)
	end

	if #msg > _max then
		
		while #msg ~= 0 do
			lastChar = msg:sub(_max, _max)

			if lastChar == "" and #msg <= _max then 
				queue[#queue+1] = msg
				msg = ""
				break

			elseif lastChar ~= " " then
				local decrement = _max
				local isWhiteSpace = false
				
				while not isWhiteSpace do
					if msg:sub(decrement, decrement) == " " then
						queue[#queue+1] = msg:sub(0, decrement)
						msg = msg:sub(decrement+1)
						isWhiteSpace = true

					else					
						decrement = decrement - 1
					end
					
				end
			
			else
				queue[#queue+1] = msg:sub(0, _max)
				msg = msg:sub(_max+1)
			end
			
		end		
	
	elseif msg == "" then
		return UI.editBox:ClearFocus()
	
	else
		queue[1] = msg		
	end
	

	for i, v in ipairs(queue) do
		queue[i] = v:gsub("^%s", "")
	end
	
	
	if type(target) == "string" then
		for i, v in ipairs(queue) do
			SendChatMessage(v, ctype, nil, target)
		end
		
	else
		for i, v in ipairs(queue) do
			BNSendWhisper(target, v)
		end

	end

	session.page:Update(true)
	lastWhisperTarget = session
	
	table.insert(cache, wipe(queue))
end

local function sendEmote (msg, session)
	local target, cmd
	
	if not msg:find("%s") then
		cmd = msg:sub(2, #msg)
	else
		firstSpace = msg:find("%s")
		cmd = msg:sub(2, firstSpace)
		target = msg:sub(firstSpace, msg:find("%s", firstSpace+1)):gsub(" ", "")

	end
	
	if target == nil and not UnitName("target") and session then
		target = session:GetName()
	end
	
	DoEmote(cmd, target)
end

local function bnetFriendUpdateHandler (...)
	friendIndex = ...
	if friendIndex == nil then 
		return 
	end
	
	payload = C_BattleNet.GetFriendAccountInfo(friendIndex)
	
	local S = App:Get(payload.bnetAccountID)
	if not S then return end
	
	Core.BNetDisplayUpdate(S.tab, 
		payload.gameAccountInfo.characterName,
		payload.gameAccountInfo.clientProgram,
		payload.gameAccountInfo.isOnline
	)
end



local function replyHook ()
	if lastWhisperTarget == nil then
		return
	end	
	
	if not UI:IsShown() then
		UI:Show()
	end
	
	App:Switch(lastWhisperTarget)
	
	UI.editBox:EnableKeyboard(false)
	UI.editBox:SetFocus()
	
	C_Timer.After(0, function() UI.editBox:EnableKeyboard(true) end)
	
end


local function onTellTargetIntercept (target)
	local id = BNet_GetBNetIDAccount(target)
	local name = Ambiguate( Utils.capitalize(target), "short" )

	if id == nil then
		if target:match("-") == nil then
			fullName = string.upper(target.. "-".. playerRealm)
		else
			fullName = target:upper()
		end
		
		if UnitName("target") == name and UnitIsPlayer("target") then
			guid = UnitGUID("target")
			
		elseif UnitName("mouseover") == name and UnitIsPlayer("mouseover") then
			guid = UnitGUID("mouseover")
			
		else
			guid = Core.History:GetInfo(fullName)
		end
		
		if guid and not App:Get(name) then
			App.active[name] = Session:CreateFromID(guid, false)
			App.active[name]:Init()
			return
		end
		
		if App:Get(name) then
			return App:Switch(App:Get(name))
		end
		
		resetPointer()
		
		App.active[name] = Session:New(name, target)
		App.active[name]:Init("CHANNEL")
		temporaryPointer = name

		
	else
		if App:Get(id) then
			App:Switch(App:Get(id))
			return
		end

		App.active[id] = Session:CreateFromID(id, true)
		App.active[id]:Init()
	end
	
	UI.leftFrame.inputBox:SetText("")
	UI.leftFrame.inputBox:ClearFocus()
end


local function updateHeaderHook (editBox)
	if editBox:GetAttribute("chatType"):match("WHISPER") == nil then
		Core.G_ChatType = editBox:GetAttribute("chatType")
	end	
end


local function extractTellTargetHook (editBox, msg)
	local target = msg:match("%s*(.*)")
	
	if ( not target or not target:find("%s") ) then
		return false;
	end
	
	if target:sub(1, 1) == "|" then
		return false
	end
	
	while target:find("%s") do
		target = target:match("(.+)%s+[^%s]*")
		if #GetAutoCompleteResults(target, 1, 0, true, AUTOCOMPLETE_LIST.ALL.include,
			AUTOCOMPLETE_LIST.ALL.exclude) > 0 then 
			break
		end
		
	end

	onTellTargetIntercept(target)
	
	UI:Show()
	editBox:Hide()
	
end



local G_GetActiveWindow = ChatEdit_GetActiveWindow
-- replacing the global function here while keeping a local reference.
function ChatEdit_GetActiveWindow ()
	if UI.editBox:HasFocus() then
		return UI.editBox
		
	else
		return G_GetActiveWindow()
	end
end


local function onPortraitsUpdated ()
	if #Core.orderedTabs == 0 or InCombatLockdown() or not IsInGroup(LE_PARTY_CATEGORY_HOME) then
		return
	end
	
	for i=1, GetNumGroupMembers(LE_PARTY_CATEGORY_HOME)-1 do
		local unit = "party".. tostring(i)
		local name = UnitName(unit)
		
		if App:Get(name) and UnitIsVisible(unit) then
			App.active[name]:UpdatePortrait(unit)
		end
	end	
end



-------------------------------------------------------------------
------------------------ Hook dook --------------------------------
-------------------------------------------------------------------



UI.editBox:HookScript("OnTabPressed", function(self)
	if #Core.orderedTabs <= 1 then
		return
	end
	
	for N, S in pairs(App.active) do
		if S.hasUnread then
			App:Switch(S)
			return
		end
	end
	
	local tabTarget
	
	if tabTraverseIndex == #Core.orderedTabs then
		tabTarget = 1
		
	elseif tabTraverseIndex+1 <= #Core.orderedTabs then
		tabTarget = tabTraverseIndex + 1
	end
	
	for N, S in pairs(App.active) do
		if S.tab.tabIndex == tabTarget then
			App:Switch(S)
			break
		end
	end
	
end)


UI.editBox:HookScript("OnSpacePressed", function(this)
	local sample = this:GetText()
	
	if (sample:sub(1, 1) ~= "/") or not SESMSG_Profile["NearbyChatEnabled"] or (selectedSession ~= nil and not selectedSession.isCustom) then
		return
	end
	
	if not selectedSession then
		App:AddCustomChannel("Nearby")
	end
	
	local ctype = sample:sub(2, sample:find("%s")-1)
	
	if selectedSession.isCustom then
		local S = App:Get("NEARBY")
		
		if (ctype == "me") or (ctype == "em") or (ctype == "e") or (ctype == "emote") then
			S.chatType = "EMOTE"
			sample = playerCharacter.. sample:sub(#ctype+3, #sample)
			
		elseif (ctype == "s") or (ctype == "say") then
			S.chatType = "SAY"
			sample = sample:sub(#ctype+3, #sample)
		end
		
		S:UpdateTitle()
		this:SetText(sample)
	end
	
	this:SetTextColor(selectedSession:GetRGB())
end)


UI.editBox:HookScript("OnTextChanged", function(this, byUser)
	if not selectedSession or not byUser or this:GetText() == "" then return end

	if selectedSession.isUsingSM then
		local S = selectedSession

		if time() >= S.typeInform_lastSent then
			if type(S.targetID) ~= "number" then
				C_ChatInfo.SendAddonMessage("SessionsMSGR", "TI01", "WHISPER", S.targetID)
				S.typeInform_lastSent = time()+5
			end
		else
			return
		end
	end
end)


UI:SetScript("OnEvent", function(self, e, ...)
	if e:match("WHISPER") then
		chatMsgWhisperHandler(e:gsub("CHAT_MSG_", ""), ...)
	
	elseif e == "BN_FRIEND_INFO_CHANGED" then
		bnetFriendUpdateHandler(...)
		
	elseif e == "CHAT_MSG_GUILD" and SESMSG_Profile["GuildEnabled"] then
		chatMsgChannelHandler(e, ...)
	
	elseif e == "CHAT_MSG_TEXT_EMOTE" or e == "CHAT_MSG_SAY" or e == "CHAT_MSG_EMOTE" or e == "CHAT_MSG_YELL" then
		chatMsgCustomHandler(e:gsub("CHAT_MSG_", ""), ...)
	
	elseif e == "GROUP_JOINED" and not App:Get("PARTY") then
		App:AddChannel("Party")
		
	elseif e == "CHAT_MSG_PARTY" or e == "CHAT_MSG_PARTY_LEADER" then
		chatMsgPartyHandler(e, ...)	
		
	elseif e == "CHAT_MSG_RESTRICTED" then
		chatMsgRestrictedHandler(...)
	
	elseif e == "MODIFIER_STATE_CHANGED" and UI:IsShown() then
		local key, state = ...
		if key ~= "LCTRL" then return end
		
		if state == 1 then
			if selectedSession ~= nil then
				selectedSession.page:SetTextCopyable(true)
			end
			
			UI.resizeBtn:Show()
			
		else
			if selectedSession ~= nil then
				selectedSession.page:SetTextCopyable(false)
			end
			
			UI:StopMovingOrSizing()
			UI.resizeBtn:Hide()
		end
		
	elseif e == "PLAYER_TARGET_CHANGED" then
		if not UnitName("target") then
			return
		end
		
		local name = Ambiguate(UnitName("target"), "short")
		
		if App:Get(name) and UnitIsVisible("target") then
			App.active[name]:UpdatePortrait("target")
		end
	
	elseif e == "GROUP_ROSTER_UPDATE" then
		if IsInRaid() then
			return
		end
		
		onPortraitsUpdated()
		
	elseif e == "PORTRAITS_UPDATED" then
		onPortraitsUpdated()
		
	elseif e == "WHO_LIST_UPDATE" then
		App:OnWhoReceived()
		
	elseif e == "ZONE_CHANGED" and App:Get("NEARBY") then
		App:Get("NEARBY"):UpdateTitle()
		
	elseif e == "CHAT_MSG_ADDON" then
		if select(1, ...) ~= "SessionsMSGR" then 
			return
		end
		
		local _, cmd, _, sender = ...
		local S = App:Get(Ambiguate(sender, "short"))
		
		if not S then 
			return 
		elseif cmd == "TI01" then
			S:SetTyping()
		else
			S:CommCheck("IN")
		end
	
		
	end
	
	
end)


UI.editBox:HookScript("OnEnterPressed", function(self, ...)
	if string.sub(self:GetText(), 1, 1) == "/" then
		sendEmote(self:GetText(), selectedSession)
	
	elseif self:HasFocus() and selectedSession ~= nil then
		sendMessage(self:GetText(), selectedSession)
	end
	
	UI:SetSliderValues()
	self:SetText("")
	self:ClearFocus()
	
end)


UI:HookScript("OnShow", function(self)
	Core.hub:StopNotifying()
	
	if selectedSession == nil then
		C_Timer.After(0, function() UI.leftFrame.inputBox:SetFocus() end)
	end
	
	for N, S in pairs(App.active) do
		if S.hasUnread then
			S:StartGlowing()
		end
	end	
end)


UI:HookScript("OnHide", function(self)
	for N, S in pairs(App.active) do
		if S.hasUnread then
			S:StopGlowing()
			
			if not Core.hub.glow.animation:IsPlaying() then
				Core.hub:Notify()
			end
			
			Core.hub:Toast("You have unread messages!")
		end
	end
end)


UI.leftFrame.inputBox:SetScript("OnEnterPressed", function(self)
	if self:GetText() ~= "" then
		onTellTargetIntercept(self:GetText())
	end
	
	self:SetText("") ; self:ClearFocus()
end)

UI.leftFrame.inputBox:HookScript("OnTextChanged", function(self, byUser)
	if not byUser then
		return
	end
	
	local text = Core.utils.capitalize(self:GetText())
	
	Core.suggestionFrame:ClearList()
	
	if text == "" then 
		return Core.suggestionFrame:Hide() 
	end
	
	Core.suggestionFrame:Show()
	
	local results = GetAutoCompleteResults(text, 4, self:GetCursorPosition(), true, AUTOCOMPLETE_FLAG_ONLINE, AUTOCOMPLETE_FLAG_NONE)

	for i, v in ipairs(results) do
		if Core.suggestionFrame:HasSpace() then
			Core.suggestionFrame:Add(Ambiguate(v.name, "none"), function() onTellTargetIntercept(v.name) end)
		end
	end
	
end)


hooksecurefunc("ChatEdit_ExtractTellTarget", extractTellTargetHook)
hooksecurefunc("ChatEdit_UpdateHeader", updateHeaderHook)
hooksecurefunc("ChatFrame_ReplyTell", replyHook)
hooksecurefunc("ChatFrame_ReplyTell2", replyHook)


SlashCmdList["SESSIONS_TALKTO"] = function(msg)
	onTellTargetIntercept(msg)
	UI:Show()
end

SLASH_SESSIONS_TALKTO1 = "/talkto"
SLASH_SESSIONS_TALKTO2 = "/tt"

Core.App = App

function Core.LoadChannels ()
	if IsInGroup(LE_PARTY_CATEGORY_HOME) and SESMSG_Profile["PartyEnabled"] and not App:Get("PARTY") then
		App:AddChannel("Party")
	end
	
	if IsInGuild() and SESMSG_Profile["GuildEnabled"] and not App:Get("GUILD") then
		App:AddChannel("Guild")
	end
end


function Core.UpdatePlayerInfo ()
	playerCharacter = UnitName("Player")
	playerRealm = GetRealmName():gsub(" ", "")
	playerName = Ambiguate(select(2, BNGetInfo()), "short")
end



